#include <stdio.h>

#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#elif defined(__linux__)
#include <sys/sysinfo.h>
#elif defined(__APPLE__)
#include <sys/sysctl.h>
#include <time.h>
#include <sys/time.h>
#include <stdlib.h>
#endif

unsigned long long get_uptime_millis();
void get_load_averages(double loadavg[3]);

int main() {
    unsigned long long uptimeMillis = get_uptime_millis();
    unsigned long long seconds = uptimeMillis / 1000;
    unsigned long long minutes = seconds / 60;
    unsigned long long hours = minutes / 60;
    unsigned long long days = hours / 24;

    seconds %= 60;
    minutes %= 60;
    hours %= 24;

    double loadavg[3];
    get_load_averages(loadavg);

    printf("Uptime: %llu days, %llu hours, %llu minutes, %llu seconds\n", days, hours, minutes, seconds);
    printf("Load averages: %.2f, %.2f, %.2f\n", loadavg[0], loadavg[1], loadavg[2]);

    return 0;
}

unsigned long long get_uptime_millis() {
#if defined(_WIN32) || defined(_WIN64)
    return GetTickCount64();
#elif defined(__linux__)
    struct sysinfo s_info;
    if (sysinfo(&s_info) != 0) {
        return 0;
    }
    return (unsigned long long)s_info.uptime * 1000;
#elif defined(__APPLE__)
    struct timeval boottime;
    int mib[2] = {CTL_KERN, KERN_BOOTTIME};
    size_t size = sizeof(boottime);
    if (sysctl(mib, 2, &boottime, &size, NULL, 0) != 0) {
        return 0;
    }
    time_t bsec = boottime.tv_sec, csec = time(NULL);
    return (unsigned long long)(csec - bsec) * 1000;
#else
    return 0;
#endif
}

void get_load_averages(double loadavg[3]) {
#if defined(_WIN32) || defined(_WIN64)
    // Windows does not have a straightforward way to get load averages.
    // You would typically need to calculate it manually based on system performance counters.
    loadavg[0] = 0.0;
    loadavg[1] = 0.0;
    loadavg[2] = 0.0;
#elif defined(__linux__)
    struct sysinfo s_info;
    if (sysinfo(&s_info) == 0) {
        loadavg[0] = s_info.loads[0] / 65536.0;
        loadavg[1] = s_info.loads[1] / 65536.0;
        loadavg[2] = s_info.loads[2] / 65536.0;
    } else {
        loadavg[0] = loadavg[1] = loadavg[2] = 0.0;
    }
#elif defined(__APPLE__)
    if (getloadavg(loadavg, 3) == -1) {
        loadavg[0] = loadavg[1] = loadavg[2] = 0.0;
    }
#else
    loadavg[0] = loadavg[1] = loadavg[2] = 0.0;
#endif
}
